package com.mortals.xhx.module.datum.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.OrderCol;
import com.mortals.xhx.common.code.ComponentEnum;
import com.mortals.xhx.common.code.DataTypeEnum;
import com.mortals.xhx.common.code.FieldTypeEnum;
import com.mortals.xhx.module.datum.model.DatumInfoFieldQuery;
import com.mortals.xhx.module.matter.model.MatterDatumPrintEntity;
import com.mortals.xhx.module.matter.model.MatterDatumPrintQuery;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.datum.dao.DatumInfoFieldDao;
import com.mortals.xhx.module.datum.model.DatumInfoFieldEntity;
import com.mortals.xhx.module.datum.service.DatumInfoFieldService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * DatumInfoFieldService
 * 填单用户信息字段 service实现
 *
 * @author zxfei
 * @date 2024-04-23
 */
@Service("datumInfoFieldService")
@Slf4j
public class DatumInfoFieldServiceImpl extends AbstractCRUDServiceImpl<DatumInfoFieldDao, DatumInfoFieldEntity, Long> implements DatumInfoFieldService {

    @Override
    public Rest<List<DatumInfoFieldEntity>> parseFormContent(MatterDatumPrintEntity matterDatumPrintEntity, Context context) {
        List<DatumInfoFieldEntity> infoFieldlist = new ArrayList<DatumInfoFieldEntity>();
      try {
        JSONObject json = JSON.parseObject(matterDatumPrintEntity.getFormContent());
        for (String key : json.keySet()) {
            DatumInfoFieldEntity infoField = new DatumInfoFieldEntity();
            infoField.setDatumPrintId(matterDatumPrintEntity.getId());
            infoField.setIdCard(matterDatumPrintEntity.getIdCard());
            //通过key类型判断属性类型
            List<String> split = StrUtil.split(key, "_");
            String value = json.getString(key);
            //单行输入框
            boolean input = StrUtil.startWith(key, ComponentEnum.INPUT.getValue());
            if (input && split.size() > 2) {
                infoField.setFieldCode(split.get(2));
                infoField.setFieldName(split.get(2));
                infoField.setFieldType(FieldTypeEnum.单行输入框.getDesc());
                infoField.setFieldValue(value);
                infoField.setDataType(DataTypeEnum.字符串.getDesc());
            }
            //文本输入框
            boolean texxarea = StrUtil.startWith(key, ComponentEnum.TEXTAREA.getValue());
            if (texxarea && split.size() > 2) {
                infoField.setFieldCode(split.get(2));
                infoField.setFieldName(split.get(2));
                infoField.setFieldType(FieldTypeEnum.多项选择框.getDesc());
                infoField.setFieldValue(value);
                infoField.setDataType(DataTypeEnum.字符串.getDesc());
            }
            //单选框
            boolean radio = StrUtil.startWith(key, ComponentEnum.RADIO.getValue());
            if (radio && split.size() > 2) {
                infoField.setFieldCode(split.get(2));
                infoField.setFieldName(split.get(2));
                infoField.setFieldType(FieldTypeEnum.单项选择框.getDesc());
                infoField.setFieldValue(value);
                infoField.setDataType(DataTypeEnum.字符串.getDesc());
            }
            //下拉选项框
            boolean select = StrUtil.startWith(key, ComponentEnum.SELECT.getValue());
            if (select && split.size() > 2) {
                infoField.setFieldCode(split.get(2));
                infoField.setFieldName(split.get(2));
                infoField.setFieldType(FieldTypeEnum.单项选择框.getDesc());
                infoField.setFieldValue(value);
                infoField.setDataType(DataTypeEnum.字符串.getDesc());
            }
            //复选框
            boolean checkBok = StrUtil.startWith(key, ComponentEnum.CHECKBOX.getValue());
            if (checkBok && split.size() > 2) {
                infoField.setFieldCode(split.get(2));
                infoField.setFieldName(split.get(2));
                infoField.setFieldType(FieldTypeEnum.多项选择框.getDesc());
                infoField.setFieldValue(value);
                infoField.setDataType(DataTypeEnum.数组.getDesc());
            }
            //时间
            boolean date = StrUtil.startWith(key, ComponentEnum.DATE.getValue());
            if (date && split.size() > 2) {
                infoField.setFieldCode(split.get(2));
                infoField.setFieldName(split.get(2));
                infoField.setFieldType(FieldTypeEnum.日期选择框.getDesc());
                infoField.setFieldValue(value);
                infoField.setDataType(DataTypeEnum.字符串.getDesc());
            }

            //横向列表
            boolean list = StrUtil.startWith(key, ComponentEnum.LIST.getValue());
            if (list && split.size() > 2) {
                infoField.setFieldCode(split.get(2));
                infoField.setFieldName(split.get(2));
                infoField.setFieldType(FieldTypeEnum.横向列表.getDesc());
                infoField.setFieldValue(value);
                infoField.setDataType(DataTypeEnum.数组.getDesc());
            }
            //表格
            boolean table = StrUtil.startWith(key, ComponentEnum.DYNAMIC_TABLE.getValue());
            if (table && split.size() > 2) {
                infoField.setFieldCode(split.get(2));
                infoField.setFieldName(split.get(2));
                infoField.setFieldType(FieldTypeEnum.表格.getDesc());
                infoField.setFieldValue(value);
                infoField.setDataType(DataTypeEnum.数组.getDesc());
            }
            infoFieldlist.add(infoField);
        }

      }catch (Exception e){
          log.error("解析json错误，json=====>{}",matterDatumPrintEntity.getFormContent());
      }
        return Rest.ok("解析保存成功！", infoFieldlist);
    }

    @Override
    public Rest<List<DatumInfoFieldEntity>> distinctList(DatumInfoFieldQuery datumInfoFieldQuery, Context context) {
        datumInfoFieldQuery.setOrderColList(Arrays.asList(new OrderCol("createTime", OrderCol.DESCENDING)));
        Map<String, List<DatumInfoFieldEntity>> collect = this.find(datumInfoFieldQuery, context).stream().collect(Collectors.groupingBy(x -> x.getFieldName()));

        List<DatumInfoFieldEntity> distiinctList = collect.entrySet().stream().map(item -> {
            List<DatumInfoFieldEntity> value = item.getValue();
            if (!ObjectUtils.isEmpty(value)) {
                return value.get(0);
            } else {
                return null;
            }
        }).filter(f -> f != null).collect(Collectors.toList());
        return Rest.ok("获取列表成功！",distiinctList);
    }


    public static void main(String[] args) {
        String json = "{\n" +
                "    \"ck_1_类别_<新参保登记-暂停登记-注销登记-拆分-合并-分立>\": [\n" +
                "        \"新参保登记\"\n" +
                "    ],\n" +
                "    \"i_2_单位名称\": \"蜻蜓和\",\n" +
                "    \"i_3_现统一社会信用代码\": \"34534543\",\n" +
                "    \"i_4_姓名\": \"hjs\",\n" +
                "    \"i_5_联系电话\": \"18282787877\",\n" +
                "    \"i_6_身份证号码\": \"51192319991118605X\",\n" +
                "    \"i_7_开户行\": \"1234213123\",\n" +
                "    \"i_8_户名\": \"hjg\",\n" +
                "    \"i_9_银行账号\": \"6745645645\",\n" +
                "    \"i_10_姓名\": \"hjs\",\n" +
                "    \"i_11_手机号码\": \"16434534534543\",\n" +
                "    \"i_12_联系电话\": \"23423423432\",\n" +
                "    \"ck_13_参保险种_<职工基本医疗保险-生育保险-补充医疗保险->其他（___________）\": [\n" +
                "        \"职工基本医疗保险\"\n" +
                "    ],\n" +
                "    \"date_14_参保起始时间\": \"2023 年 02 月 22 日\",\n" +
                "    \"date_15_日期\": \"2023 年 02 月 14 日\"\n" +
                "}";


        JSONObject jsonObject = JSON.parseObject(json);


        String string = jsonObject.getString("ck_13_参保险种_<职工基本医疗保险-生育保险-补充医疗保险->其他（___________）");


        System.out.println(string);
    }

}